@@ -1,10 +1,8 @@
 import logging
 
-from six import string_types
-
-from .fields import IntegerField, EnumField, EnumListField, DateField, DateTimeField, EWSElementField, \
-    MONTHS, WEEK_NUMBERS, WEEKDAYS
-from .properties import EWSElement, IdChangeKeyMixIn
+from .fields import IntegerField, EnumField, EnumListField, DateOrDateTimeField, DateTimeField, EWSElementField, \
+    IdElementField, MONTHS, WEEK_NUMBERS, WEEKDAYS
+from .properties import EWSElement, IdChangeKeyMixIn, ItemId, Fields
 
 log = logging.getLogger(__name__)
 
@@ -26,18 +24,23 @@ class Pattern(EWSElement):
     __slots__ = tuple()
 
 
+class Regeneration(Pattern):
+    """Base class for all classes implementing recurring regeneration elements"""
+    __slots__ = tuple()
+
+
 class AbsoluteYearlyPattern(Pattern):
     """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/absoluteyearlyrecurrence
     """
     ELEMENT_NAME = 'AbsoluteYearlyRecurrence'
 
-    FIELDS = [
-        # The month of the year, from 1 - 12
-        EnumField('month', field_uri='t:Month', enum=MONTHS, is_required=True),
+    FIELDS = Fields(
         # The day of month of an occurrence, in range 1 -> 31. If a particular month has less days than the day_of_month
         # value, the last day in the month is assumed
-        IntegerField('day_of_month', field_uri='t:DayOfMonth', min=1, max=31, is_required=True),
-    ]
+        IntegerField('day_of_month', field_uri='DayOfMonth', min=1, max=31, is_required=True),
+        # The month of the year, from 1 - 12
+        EnumField('month', field_uri='Month', enum=MONTHS, is_required=True),
+    )
 
     __slots__ = tuple(f.name for f in FIELDS)
 
@@ -50,17 +53,17 @@ class RelativeYearlyPattern(Pattern):
     """
     ELEMENT_NAME = 'RelativeYearlyRecurrence'
 
-    FIELDS = [
+    FIELDS = Fields(
         # The weekday of the occurrence, as a valid ISO 8601 weekday number in range 1 -> 7 (1 being Monday).
         # Alternatively, the weekday can be one of the DAY (or 8), WEEK_DAY (or 9) or WEEKEND_DAY (or 10) consts which
         # is interpreted as the first day, weekday, or weekend day in the month, respectively.
-        EnumField('weekday', field_uri='t:DaysOfWeek', enum=WEEKDAYS, is_required=True),
+        EnumField('weekday', field_uri='DaysOfWeek', enum=WEEKDAYS, is_required=True),
         # Week number of the month, in range 1 -> 5. If 5 is specified, this assumes the last week of the month for
         # months that have only 4 weeks
-        EnumField('week_number', field_uri='t:DayOfWeekIndex', enum=WEEK_NUMBERS, is_required=True),
+        EnumField('week_number', field_uri='DayOfWeekIndex', enum=WEEK_NUMBERS, is_required=True),
         # The month of the year, from 1 - 12
-        EnumField('month', field_uri='t:Month', enum=MONTHS, is_required=True),
-    ]
+        EnumField('month', field_uri='Month', enum=MONTHS, is_required=True),
+    )
 
     __slots__ = tuple(f.name for f in FIELDS)
 
@@ -77,13 +80,13 @@ class AbsoluteMonthlyPattern(Pattern):
     """
     ELEMENT_NAME = 'AbsoluteMonthlyRecurrence'
 
-    FIELDS = [
+    FIELDS = Fields(
         # Interval, in months, in range 1 -> 99
-        IntegerField('interval', field_uri='t:Interval', min=1, max=99, is_required=True),
+        IntegerField('interval', field_uri='Interval', min=1, max=99, is_required=True),
         # The day of month of an occurrence, in range 1 -> 31. If a particular month has less days than the day_of_month
         # value, the last day in the month is assumed
-        IntegerField('day_of_month', field_uri='t:DayOfMonth', min=1, max=31, is_required=True),
-    ]
+        IntegerField('day_of_month', field_uri='DayOfMonth', min=1, max=31, is_required=True),
+    )
 
     __slots__ = tuple(f.name for f in FIELDS)
 
@@ -96,17 +99,17 @@ class RelativeMonthlyPattern(Pattern):
     """
     ELEMENT_NAME = 'RelativeMonthlyRecurrence'
 
-    FIELDS = [
+    FIELDS = Fields(
         # Interval, in months, in range 1 -> 99
-        IntegerField('interval', field_uri='t:Interval', min=1, max=99, is_required=True),
+        IntegerField('interval', field_uri='Interval', min=1, max=99, is_required=True),
         # The weekday of the occurrence, as a valid ISO 8601 weekday number in range 1 -> 7 (1 being Monday).
         # Alternatively, the weekday can be one of the DAY (or 8), WEEK_DAY (or 9) or WEEKEND_DAY (or 10) consts which
         # is interpreted as the first day, weekday, or weekend day in the month, respectively.
-        EnumField('weekday', field_uri='t:DaysOfWeek', enum=WEEKDAYS, is_required=True),
+        EnumField('weekday', field_uri='DaysOfWeek', enum=WEEKDAYS, is_required=True),
         # Week number of the month, in range 1 -> 5. If 5 is specified, this assumes the last week of the month for
         # months that have only 4 weeks.
-        EnumField('week_number', field_uri='t:DayOfWeekIndex', enum=WEEK_NUMBERS, is_required=True),
-    ]
+        EnumField('week_number', field_uri='DayOfWeekIndex', enum=WEEK_NUMBERS, is_required=True),
+    )
 
     __slots__ = tuple(f.name for f in FIELDS)
 
@@ -119,23 +122,22 @@ class RelativeMonthlyPattern(Pattern):
 
 
 class WeeklyPattern(Pattern):
-    """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/weeklyrecurrence
-    """
+    """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/weeklyrecurrence"""
     ELEMENT_NAME = 'WeeklyRecurrence'
 
-    FIELDS = [
+    FIELDS = Fields(
         # Interval, in weeks, in range 1 -> 99
-        IntegerField('interval', field_uri='t:Interval', min=1, max=99, is_required=True),
+        IntegerField('interval', field_uri='Interval', min=1, max=99, is_required=True),
         # List of valid ISO 8601 weekdays, as list of numbers in range 1 -> 7 (1 being Monday)
-        EnumListField('weekdays', field_uri='t:DaysOfWeek', enum=WEEKDAYS, is_required=True),
+        EnumListField('weekdays', field_uri='DaysOfWeek', enum=WEEKDAYS, is_required=True),
         # The first day of the week. Defaults to Monday
-        EnumField('first_day_of_week', field_uri='t:FirstDayOfWeek', enum=WEEKDAYS, default=1, is_required=True),
-    ]
+        EnumField('first_day_of_week', field_uri='FirstDayOfWeek', enum=WEEKDAYS, default=1, is_required=True),
+    )
 
     __slots__ = tuple(f.name for f in FIELDS)
 
     def __str__(self):
-        if isinstance(self.weekdays, string_types):
+        if isinstance(self.weekdays, str):
             weekdays = [self.weekdays]
         elif isinstance(self.weekdays, int):
             weekdays = [_weekday_to_str(self.weekdays)]
@@ -147,14 +149,13 @@ class WeeklyPattern(Pattern):
 
 
 class DailyPattern(Pattern):
-    """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/dailyrecurrence
-    """
+    """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/dailyrecurrence"""
     ELEMENT_NAME = 'DailyRecurrence'
 
-    FIELDS = [
+    FIELDS = Fields(
         # Interval, in days, in range 1 -> 999
-        IntegerField('interval', field_uri='t:Interval', min=1, max=999, is_required=True),
-    ]
+        IntegerField('interval', field_uri='Interval', min=1, max=999, is_required=True),
+    )
 
     __slots__ = tuple(f.name for f in FIELDS)
 
@@ -162,68 +163,136 @@ class DailyPattern(Pattern):
         return 'Occurs every %s day(s)' % self.interval
 
 
+class YearlyRegeneration(Regeneration):
+    """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/yearlyregeneration"""
+    ELEMENT_NAME = 'YearlyRegeneration'
+
+    FIELDS = Fields(
+        # Interval, in years
+        IntegerField('interval', field_uri='Interval', min=1, is_required=True),
+    )
+
+    __slots__ = tuple(f.name for f in FIELDS)
+
+    def __str__(self):
+        return 'Regenerates every %s year(s)' % self.interval
+
+
+class MonthlyRegeneration(Regeneration):
+    """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/monthlyregeneration"""
+    ELEMENT_NAME = 'MonthlyRegeneration'
+
+    FIELDS = Fields(
+        # Interval, in months
+        IntegerField('interval', field_uri='Interval', min=1, is_required=True),
+    )
+
+    __slots__ = tuple(f.name for f in FIELDS)
+
+    def __str__(self):
+        return 'Regenerates every %s month(s)' % self.interval
+
+
+class WeeklyRegeneration(Regeneration):
+    """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/weeklyregeneration"""
+    ELEMENT_NAME = 'WeeklyRegeneration'
+
+    FIELDS = Fields(
+        # Interval, in weeks
+        IntegerField('interval', field_uri='Interval', min=1, is_required=True),
+    )
+
+    __slots__ = tuple(f.name for f in FIELDS)
+
+    def __str__(self):
+        return 'Regenerates every %s week(s)' % self.interval
+
+
+class DailyRegeneration(Regeneration):
+    """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/dailyregeneration"""
+    ELEMENT_NAME = 'DailyRegeneration'
+
+    FIELDS = Fields(
+        # Interval, in days
+        IntegerField('interval', field_uri='Interval', min=1, is_required=True),
+    )
+
+    __slots__ = tuple(f.name for f in FIELDS)
+
+    def __str__(self):
+        return 'Regenerates every %s day(s)' % self.interval
+
+
 class Boundary(EWSElement):
     """Base class for all classes implementing recurring boundary elements"""
     __slots__ = tuple()
 
 
 class NoEndPattern(Boundary):
-    """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/noendrecurrence
-    """
+    """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/noendrecurrence"""
     ELEMENT_NAME = 'NoEndRecurrence'
 
-    FIELDS = [
-        # Start date, as EWSDate
-        DateField('start', field_uri='t:StartDate', is_required=True),
-    ]
+    FIELDS = Fields(
+        # Start date, as EWSDate or EWSDateTime
+        DateOrDateTimeField('start', field_uri='StartDate', is_required=True),
+    )
 
     __slots__ = tuple(f.name for f in FIELDS)
 
+    def __str__(self):
+        return 'Starts on %s' % self.start
+
 
 class EndDatePattern(Boundary):
-    """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/enddaterecurrence
-    """
+    """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/enddaterecurrence"""
     ELEMENT_NAME = 'EndDateRecurrence'
 
-    FIELDS = [
-        # Start date, as EWSDate
-        DateField('start', field_uri='t:StartDate', is_required=True),
+    FIELDS = Fields(
+        # Start date, as EWSDate or EWSDateTime
+        DateOrDateTimeField('start', field_uri='StartDate', is_required=True),
         # End date, as EWSDate
-        DateField('end', field_uri='t:EndDate', is_required=True),
-    ]
+        DateOrDateTimeField('end', field_uri='EndDate', is_required=True),
+    )
 
     __slots__ = tuple(f.name for f in FIELDS)
 
+    def __str__(self):
+        return 'Starts on %s, ends on %s' % (self.start, self.end)
+
 
 class NumberedPattern(Boundary):
     """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/numberedrecurrence"""
     ELEMENT_NAME = 'NumberedRecurrence'
 
-    FIELDS = [
-        # Start date, as EWSDate
-        DateField('start', field_uri='t:StartDate', is_required=True),
+    FIELDS = Fields(
+        # Start date, as EWSDate or EWSDateTime
+        DateOrDateTimeField('start', field_uri='StartDate', is_required=True),
         # The number of occurrences in this pattern, in range 1 -> 999
-        IntegerField('number', field_uri='t:NumberOfOccurrences', min=1, max=999, is_required=True),
-    ]
+        IntegerField('number', field_uri='NumberOfOccurrences', min=1, max=999, is_required=True),
+    )
 
     __slots__ = tuple(f.name for f in FIELDS)
 
+    def __str__(self):
+        return 'Starts on %s and occurs %s times' % (self.start, self.number)
+
 
 class Occurrence(IdChangeKeyMixIn):
     """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/occurrence"""
     ELEMENT_NAME = 'Occurrence'
+    ID_ELEMENT_CLS = ItemId
 
-    LOCAL_FIELDS = [
+    FIELDS = Fields(
+        IdElementField('_id', field_uri='ItemId', value_cls=ID_ELEMENT_CLS),
         # The modified start time of the item, as EWSDateTime
-        DateTimeField('start', field_uri='t:Start'),
+        DateTimeField('start', field_uri='Start'),
         # The modified end time of the item, as EWSDateTime
-        DateTimeField('end', field_uri='t:End'),
+        DateTimeField('end', field_uri='End'),
         # The original start time of the item, as EWSDateTime
-        DateTimeField('original_start', field_uri='t:OriginalStart'),
-    ]
-    FIELDS = IdChangeKeyMixIn.FIELDS + LOCAL_FIELDS
+        DateTimeField('original_start', field_uri='OriginalStart'),
+    )
 
-    __slots__ = tuple(f.name for f in LOCAL_FIELDS)
+    __slots__ = tuple(f.name for f in FIELDS)
 
 
 # Container elements:
@@ -247,16 +316,17 @@ class DeletedOccurrence(EWSElement):
     """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/deletedoccurrence"""
     ELEMENT_NAME = 'DeletedOccurrence'
 
-    FIELDS = [
+    FIELDS = Fields(
         # The modified start time of the item, as EWSDateTime
-        DateTimeField('start', field_uri='t:Start'),
-    ]
+        DateTimeField('start', field_uri='Start'),
+    )
 
     __slots__ = tuple(f.name for f in FIELDS)
 
 
 PATTERN_CLASSES = AbsoluteYearlyPattern, RelativeYearlyPattern, AbsoluteMonthlyPattern, RelativeMonthlyPattern, \
                    WeeklyPattern, DailyPattern
+REGENERATION_CLASSES = YearlyRegeneration, MonthlyRegeneration, WeeklyRegeneration, DailyRegeneration
 BOUNDARY_CLASSES = NoEndPattern, EndDatePattern, NumberedPattern
 
 
@@ -265,10 +335,11 @@ class Recurrence(EWSElement):
     """
     ELEMENT_NAME = 'Recurrence'
 
-    FIELDS = [
+    FIELDS = Fields(
         EWSElementField('pattern', value_cls=Pattern),
         EWSElementField('boundary', value_cls=Boundary),
-    ]
+    )
+    PATTERN_CLASSES = PATTERN_CLASSES
 
     __slots__ = tuple(f.name for f in FIELDS)
 
@@ -288,11 +359,11 @@ class Recurrence(EWSElement):
                 kwargs['boundary'] = NumberedPattern(start=start, number=number)
             else:
                 raise ValueError("Unsupported 'start', 'end', 'number' combination")
-        super(Recurrence, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
     @classmethod
     def from_xml(cls, elem, account):
-        for pattern_cls in PATTERN_CLASSES:
+        for pattern_cls in cls.PATTERN_CLASSES:
             pattern_elem = elem.find(pattern_cls.response_tag())
             if pattern_elem is None:
                 continue
@@ -312,3 +383,11 @@ class Recurrence(EWSElement):
 
     def __str__(self):
         return 'Pattern: %s, Boundary: %s' % (self.pattern, self.boundary)
+
+
+class TaskRecurrence(Recurrence):
+    """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/recurrence-taskrecurrencetype
+    """
+    PATTERN_CLASSES = PATTERN_CLASSES + REGENERATION_CLASSES
+
+    __slots__ = tuple()
